==> rayasdk-1.1.0.dev13.tar.gz: rayasdk-1.1.0.dev13/rayasdk/sshKeyGen.py <==
# Copyright 2020 Unlimited Robotics

import os
from pathlib import Path
import sys
import subprocess
import glob
import paramiko


import rayasdk.constants as constants
from rayasdk.logger import log_error, log_info, log_verbose, log_warning

# Constants
CUR_USER_HOME = os.path.expanduser('~')
PLATFORM = sys.platform


class SshKeyGen:

    def __init__(self, user, host, port=22):
        if PLATFORM == 'windows':
            # TODO implement this on ws
            self.PRIV_SSH_DIR = f'{CUR_USER_HOME}/.ssh'
        elif PLATFORM == 'linux' or PLATFORM == 'darwin':
            self.PRIV_SSH_DIR = f'{CUR_USER_HOME}/.ssh'  
        else:
            raise Exception('System Not Found')
        
        if not os.path.exists(self.PRIV_SSH_DIR):
            if not self.__create_ssh_directory():
                raise Exception('Failed to create SSH directory')
            
        try:
            self.__push_key(user=user, host=host, port=port)
        except Exception as error:
            self.check_ssh_error(error=str(error), host=host, port=port)
                

    def __create_ssh_directory():
        ssh_directory = os.path.expanduser('~/.ssh')
        try:
            subprocess.run(['mkdir', '-m', '700', ssh_directory], check=True)
            return True
        except subprocess.CalledProcessError as e:
            return False


    def __key_present(self):
        '''Checks to see if there is an key already present. Returns a bool.'''
        matching_files = glob.glob(os.path.join(
                constants.UR_HOME, 
                constants.SSH_KEY_PRIV
            ))
        if matching_files:
            return True
        else:
            return False


    def __gen_key(self):
        '''Generate a SSH Key.'''
        os.chdir(self.PRIV_SSH_DIR)
        if self.__key_present():
            return False
        else:
            # Generate private key
            key = paramiko.RSAKey.generate(2048)
            key.write_private_key_file(constants.SSH_KEY_PRIV)
            with open(constants.SSH_KEY_PUB, 'w') as f:
                f.write(f'ssh-rsa {key.get_base64()}')
            os.chmod(constants.SSH_KEY_PUB, 0o600)
            return True


    def __push_key(self, user, host, port=22):
        '''Push an SSH Key to a remote server.'''
        os.chdir(self.PRIV_SSH_DIR)
        if not self.__key_present():
            self.__gen_key()
            
        log_verbose('SSH key found. Pushing key to remote server')
        command = f'ssh-copy-id '
        command += f'-i {constants.SSH_KEY_PUB} -p {port} {user}@{host}'
        ssh_copy_process = subprocess.Popen(command,
                                            shell=True,
                                            stdin=subprocess.PIPE,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)
        out, err = ssh_copy_process.communicate()
        if ssh_copy_process.returncode != 0:
            raise Exception(err.decode())


    @staticmethod
    def remote_host_change(host, port):
        command = (('ssh-keygen '
                    f'-f "{Path.home()/".ssh"/"known_hosts"}" '
                    f'-R "[{host}]:{port}"'))
        return subprocess.run(
            command,
            shell=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL).check_returncode()
    
    
    @staticmethod
    def check_ssh_error(error, host, port):
        if 'REMOTE HOST IDENTIFICATION HAS CHANGED' in error:
            log_warning(
                ('Remote Host Identification Has Changed, '
                'run \'rayasdk connect again\'.'))
            SshKeyGen.remote_host_change(host=host,port=port)
        elif 'No route to host' in error:
            raise Exception(('No route to host.'))
        elif 'Connection refused' in error:
            raise Exception(
                ('Connection refused. Check that the Ra-Ya system is running.'))
        elif 'Host key verification failed':
            raise Exception(
                ('Host key verification failed, '
                'run \'rayasdk connect\'.'))
        else:
            raise Exception(error)

==> rayasdk-1.1.1.dev0.tar.gz: rayasdk-1.1.1.dev0/rayasdk/sshKeyGen.py <==
# Copyright 2020 Unlimited Robotics

import os
from pathlib import Path
import sys
import subprocess
import glob
import paramiko

import rayasdk.constants as constants
from rayasdk.logger import log_verbose, log_warning

# Constants
CUR_USER_HOME = os.path.expanduser('~')
PLATFORM = sys.platform


class SshKeyGen:

    def __init__(self, user, host, port=22):
        if PLATFORM == 'windows':
            # TODO implement this on ws
            self.PRIV_SSH_DIR = f'{CUR_USER_HOME}/.ssh'
        elif PLATFORM == 'linux' or PLATFORM == 'darwin':
            self.PRIV_SSH_DIR = f'{CUR_USER_HOME}/.ssh'
        else:
            raise Exception('System Not Found')

        if not os.path.exists(self.PRIV_SSH_DIR):
            if not self.__create_ssh_directory():
                raise Exception('Failed to create SSH directory')

        try:
            self.__push_key(user=user, host=host, port=port)
        except Exception as error:
            self.check_ssh_error(error=str(error), host=host, port=port)


    def __create_ssh_directory():
        ssh_directory = os.path.expanduser('~/.ssh')
        try:
            subprocess.run(['mkdir', '-m', '700', ssh_directory], check=True)
            return True
        except subprocess.CalledProcessError as e:
            return False


    def __key_present(self):
        '''Checks to see if there is an key already present. Returns a bool.'''
        matching_files = glob.glob(
            os.path.join(constants.UR_HOME, constants.SSH_KEY_PRIV))
        if matching_files:
            return True
        else:
            return False


    def __gen_key(self):
        '''Generate a SSH Key.'''
        os.chdir(self.PRIV_SSH_DIR)
        if self.__key_present():
            return False
        else:
            # Generate private key
            key = paramiko.RSAKey.generate(2048)
            key.write_private_key_file(constants.SSH_KEY_PRIV)
            with open(constants.SSH_KEY_PUB, 'w') as f:
                f.write(f'ssh-rsa {key.get_base64()}')
            os.chmod(constants.SSH_KEY_PUB, 0o600)
            return True


    def __push_key(self, user, host, port=22):
        '''Push an SSH Key to a remote server.'''
        os.chdir(self.PRIV_SSH_DIR)
        if not self.__key_present():
            self.__gen_key()

        log_verbose('SSH key found. Pushing key to remote server')
        command = f'ssh-copy-id '
        command += f'-i {constants.SSH_KEY_PUB} -p {port} {user}@{host}'
        ssh_copy_process = subprocess.Popen(command,
                                            shell=True,
                                            stdin=subprocess.PIPE,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)
        out, err = ssh_copy_process.communicate()
        if ssh_copy_process.returncode != 0:
            raise Exception(err.decode())


    @staticmethod
    def remote_host_change(host, port):
        command = (('ssh-keygen '
                    f'-f "{Path.home()/".ssh"/"known_hosts"}" '
                    f'-R "[{host}]:{port}"'))
        return subprocess.run(command,
                              shell=True,
                              stdout=subprocess.DEVNULL,
                              stderr=subprocess.DEVNULL).check_returncode()


    @staticmethod
    def check_ssh_error(error, host, port):
        if 'REMOTE HOST IDENTIFICATION HAS CHANGED' in error:
            log_warning(('Remote Host Identification Has Changed'))
            SshKeyGen.remote_host_change(host=host, port=port)
            SshKeyGen(host=host,port=port, user=constants.SSH_USER)
        elif 'No route to host' in error:
            raise Exception(('No route to host.'))
        elif 'Connection refused' in error:
            raise Exception((
                'Connection refused. Check that the Ra-Ya system is running.'))
        elif 'Host key verification failed':
            raise Exception(('Host key verification failed, '
                             'run \'rayasdk connect\'.'))
        else:
            raise Exception(error)
